import os, json, datetime, time, socket, requests
from distutils import file_util
from importlib.machinery import SourceFileLoader

_base_dir = os.path.join(os.path.expanduser('~'), '.dynipman')

def load_config():
    if not os.path.exists(_base_dir):
        os.makedirs(_base_dir)
    if not os.path.exists(os.path.join(_base_dir, 'conf')):
        config_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'conf.py')
        file_util.copy_file(config_file, os.path.join(_base_dir, 'conf'))
    confpy = SourceFileLoader('conf', os.path.join(_base_dir, 'conf')).load_module()
    return confpy

class Server():
    def __init__(self):
        config = load_config()
        
        self.config = config.SERVER
        self.key = config.SHARED_KEY
        self.addressbook = self.load_addressbook()
    
    def load_addressbook(self):
        result = {}
        try:
            with open(os.path.join(_base_dir, self.config['data_file']), 'r') as bookfile:
                result = json.loads(bookfile.read())
        except ValueError:
            pass
        finally:
            return result
        
    def save_addressbook(self):
        try:
            with open(os.path.join(_base_dir, self.config['data_file']), 'w') as bookfile:
                data = json.dumps(self.addressbook)
                bookfile.write(data)
            return True
        except ValueError:
            return False
        
    def update_address(self, name, new_info):
        self.addressbook[name] = new_info
        result = self.save_addressbook()
        if result:
            with open(os.path.join(_base_dir, 'log.txt'), 'a') as logfile:
                new_info['dtstamp'] = datetime.datetime.now().isoformat()
                data = json.dumps(new_info)+'\n'
                logfile.write(data)
        return result


class Client():
    def __init__(self):
        self.config = load_config()
        self.info = {
                     'host': socket.gethostname(),
                     'name': self.config.CLIENT['name']
                     }
        
    def report_ip(self):
        try:
            update = requests.post(self.config.SERVER['url']+'update/?code='+self.config.SHARED_KEY, data=json.dumps(self.info)).json()
            print(datetime.datetime.now())
            print(update)
            return update
        except requests.exceptions.ConnectionError:
            print(datetime.datetime.now())
            print('Connection Error!')
            print('  check config at '+os.path.join(_base_dir, 'conf'))
            print('  if the config is correct, then the server might be down.')
            return None
    
    def start(self):
        while True:
            self.report_ip()
            time.sleep(self.config.CLIENT['update_interval'])


# Change this secret to something only you know
SHARED_KEY = 'ThisIsNotASecureSecret'

SERVER = {
          'url': 'http://localhost:7883/', # Change this to your public domain host
          'port': 7883,
          'data_file': 'addressbook.json' # You can find this file in ~/.dynipman/
          }

CLIENT = {
          'name': 'my-home-ubuntu', # This is the name with which you would identify client machines
          'update_interval': 60 # Interval at which the client reports to server
          }


import json
from dynipman.components import Server
import tornado.ioloop
import tornado.web
    
running_server = Server()

def is_authorized(handler, server):
    code = handler.get_query_arguments('code')
    if len(code) > 0:
        return (code[0] == server.key)
    else:
        return False

class MainHandler(tornado.web.RequestHandler):
    def get(self):
        if is_authorized(self, running_server):
            message = '\n'+json.dumps(running_server.addressbook)
            self.write(message)
            print("Main - Authorized")
        else:
            self.write("Unauthorized")
            print("Main - UNAUTHORIZED")
        print(repr(self.request))
        print(self.request.body)
        
class UpdateHandler(tornado.web.RequestHandler):
    def post(self):
        client_data = json.loads(self.request.body.decode())
        if is_authorized(self, running_server):
            print('Update - Authorized')
            client_info = {
                           'ip': self.request.remote_ip,
                           'host': client_data['host'],
                           'name': client_data['name'],
                           }
            print("IP Update Request from "+client_info['name'])
            print('    Client host : '+client_info['host'])
            print('    Client IP   : '+client_info['ip'])
            saved = running_server.update_address(client_info['name'], client_info)
            
            if saved:
                response = { 'result': 'success',
                            'data': 'Update saved successfully',
                            }
            else:
                response = { 'result': 'failure',
                            'data': 'Failed to save data',
                            }
        else:
            print('Update - UNAUTHORIZED')
            response = { 'result': 'failure',
                         'data': 'Unauthorized Access',
                        }
        print(repr(self.request))
        print(self.request.body)
        self.write(response)
        
def make_app():
    print('========================')
    print(' starting dynipman server ')
    print('========================')
    return tornado.web.Application([ (r'/$', MainHandler),
                                     (r'/update/$', UpdateHandler), 
                                    ])
    
def run():
    app = make_app()
    app.listen(running_server.config['port'])
    tornado.ioloop.IOLoop.current().start()

if __name__ == "__main__":
    run()


from dynipman.components import Client

client = Client()

def device_info():
    print("Client host: "+client.info['host'])
    print("Client name: "+client.info['name'])
    return client.info
    
def report_ip():
    update = client.report_ip()
    return update

def show_menu():
    print('\n-Menu---------------------------')
    print('\n1. Show Device Info')
    print('2. Report IP Address to Server')
    print('\n type "exit" to exit ')
    print('--------------------------------')
    return {
            '1': device_info,
            '2': report_ip,
            }

def run():
    KEEP_ALIVE = True
    print('=HELLO==================')
    print(' starting dynipman client ')
    print('========================')
    while KEEP_ALIVE:
        menu = show_menu()
        uin = input(' >> ')
        if uin in menu:
            menu[uin]()
        KEEP_ALIVE = not (uin == 'exit')
    print('========================')
    print(' quitting dynipman client ')
    print('====================BYE=')
    return

if __name__ == "__main__":
    run()


from distutils.core import setup
VERSION = '0.0.1'
setup(
  name = 'dynipman',
  install_requires = ['tornado'],
  packages = ['dynipman',],
  package_data = {'dynipman':['scripts/*',]},
  scripts = ['dynipman/scripts/dynipman_cd','dynipman/scripts/dynipman_sd',],
  version = VERSION,
  description = 'Personal IP address name service for Developers who move around a lot',
  author = 'Kumseok Jung',
  author_email = 'jungkumseok@gmail.com',
  license = 'LICENSE.txt',
  url = 'https://github.com/jungkumseok/dynipman',
  download_url = 'https://github.com/jungkumseok/dynipman/archive/'+VERSION+'.tar.gz',
  keywords = ['remote', 'ip address', 'dns'], 
  classifiers = [],
)